@@ -1,6 +1,6 @@
 //
 // AutoProperties.cc for pekwm
-// Copyright © 2003-2008 Claes Nästén <me@pekdon.net>
+// Copyright © 2003-2009 Claes Nästén <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -632,6 +632,8 @@ AutoProperties::setDefaultTypeProperties(void)
         prop->skip = SKIP_MENUS|SKIP_FOCUS_TOGGLE|SKIP_SNAP|SKIP_PAGER|SKIP_TASKBAR;
         prop->maskAdd(AP_LAYER);
         prop->layer = LAYER_DESKTOP;
+        prop->maskAdd(AP_FOCUSABLE);
+        prop->focusable = false;
 
         _window_type_prop_map[WINDOW_TYPE_DESKTOP] = prop;
     }
@@ -649,6 +651,8 @@ AutoProperties::setDefaultTypeProperties(void)
         prop->skip = SKIP_MENUS|SKIP_FOCUS_TOGGLE|SKIP_PAGER|SKIP_TASKBAR;
         prop->maskAdd(AP_LAYER);
         prop->layer = LAYER_DOCK;
+        prop->maskAdd(AP_FOCUSABLE);
+        prop->focusable = false;
 
         _window_type_prop_map[WINDOW_TYPE_DOCK] = prop;
     }
