@@ -1,6 +1,6 @@
 //
 // Config.cc for pekwm
-// Copyright © 2002-2008 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2002-2009 Claes Nästén <me{@}pekdon{.}net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -93,6 +93,7 @@ Config::Config(void) :
         _screen_show_workspace_indicator(500), _screen_workspace_indicator_scale(16),
         _screen_place_new(true), _screen_focus_new(false),
         _screen_focus_new_child(true), _screen_honour_randr(true),
+        _screen_honour_aspectratio(true),
         _screen_placement_row(false),
         _screen_placement_ltr(true), _screen_placement_ttb(true),
         _screen_placement_offset_x(0), _screen_placement_offset_y(0),
@@ -588,6 +589,7 @@ Config::loadScreen(CfgParser::Entry *section)
     key_list.push_back(new CfgParserKeyBool("FOCUSNEW", _screen_focus_new));
     key_list.push_back(new CfgParserKeyBool("FOCUSNEWCHILD", _screen_focus_new_child, true));
     key_list.push_back(new CfgParserKeyBool("HONOURRANDR", _screen_honour_randr, true));
+    key_list.push_back(new CfgParserKeyBool("HONOURASPECTRATIO", _screen_honour_aspectratio, true));
 
     // Parse data
     section->parse_key_values(key_list.begin(), key_list.end());
@@ -842,12 +844,35 @@ Config::parseKey(const std::string &key_string, uint &mod, uint &key)
         num = tok.size() - 1;
         if ((tok[num].size() > 1) && (tok[num][0] == '#')) {
             key = strtol(tok[num].c_str() + 1, 0, 10);
+        } else if (strcasecmp(tok[num].c_str(), "ANY") == 0) {
+            // Do no matching, anything goes.
+            key = 0;
         } else {
-            key = XKeysymToKeycode(PScreen::instance()->getDpy(),
-                                   XStringToKeysym(tok[num].c_str()));
-            if (strcasecmp(tok[num].c_str(), "ANY") == 0) {
-                key = 0; // FIXME: for now, but there's no XK_Any?
+            KeySym keysym = XStringToKeysym(tok[num].c_str());
+
+            // XStringToKeysym() may fail. Perhaps we have luck after some
+            // simple transformations. First we convert the string to lowercase
+            // and try again. Then we try with only the first character in
+            // uppercase and at last we try a complete uppercase string. If all
+            // fails, we print a warning and return false.
+            if (keysym == NoSymbol) {
+                string str = tok[num];
+                Util::to_lower(str);
+                keysym = XStringToKeysym(str.c_str());
+                if (keysym == NoSymbol) {
+                    str[0] = ::toupper(str[0]);
+                    keysym = XStringToKeysym(str.c_str());
+                    if (keysym == NoSymbol) {
+                        Util::to_upper(str);
+                        keysym = XStringToKeysym(str.c_str());
+                        if (keysym == NoSymbol) {
+                            cerr << " *** WARNING: Couldn't find keysym for " << tok[num] << endl;
+                            return false;
+                        }
+                    }
+                }
             }
+            key = XKeysymToKeycode(PScreen::instance()->getDpy(), keysym);
         }
 
         // if the last token isn't an key/button, the action isn't valid
@@ -1451,9 +1476,11 @@ Config::copyConfigFiles(void)
     string autoprops_file = cfg_dir + string("/autoproperties");
     string start_file = cfg_dir + string("/start");
     string vars_file = cfg_dir + string("/vars");
+    string themes_dir = cfg_dir + string("/themes");
 
     bool cp_config, cp_keys, cp_mouse, cp_menu;
     bool cp_autoprops, cp_start, cp_vars;
+    bool make_themes = false;
     cp_config = cp_keys = cp_mouse = cp_menu = false;
     cp_autoprops = cp_start = cp_vars = false;
 
@@ -1507,7 +1534,9 @@ Config::copyConfigFiles(void)
             cp_start = true;
         if (stat(vars_file.c_str(), &stat_buf))
             cp_vars = true;
-
+        if (stat(themes_dir.c_str(), &stat_buf)) {
+            make_themes = true;
+        }
     } else { // we didn't have a ~/.pekwm directory already, lets create one
         if (mkdir(cfg_dir.c_str(), 0700)) {
             cerr << "Can't create " << cfg_dir << " directory!" << endl;
@@ -1517,6 +1546,7 @@ Config::copyConfigFiles(void)
 
         cp_config = cp_keys = cp_mouse = cp_menu = true;
         cp_autoprops = cp_start = cp_vars = true;
+        make_themes = true;
     }
 
     if (cp_config) {
@@ -1540,6 +1570,9 @@ Config::copyConfigFiles(void)
     if (cp_vars) {
         Util::copyTextFile(SYSCONFDIR "/vars", vars_file);
     }
+    if (make_themes) {
+        mkdir(themes_dir.c_str(), 0700);
+    }
 }
 
 /**
