@@ -1,6 +1,6 @@
 //
 // FrameListMenu.cc for pekwm
-// Copyright (C) 2002-2004 Claes Nasten <pekdon{@}pekdon{.}net>
+// Copyright (C) 2002-2009 Claes Nasten <pekdon{@}pekdon{.}net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -32,7 +32,7 @@ using std::map;
 //! @brief Constructor for DecorMenu.
 DecorMenu::DecorMenu(PScreen *scr, Theme *theme, ActionHandler *act,
                      const std::string &name) :
-        WORefMenu(scr, theme, "Decor Menu", name),
+        WORefMenu(scr, theme, L"Decor Menu", name),
         _act(act)
 {
     _menu_type = DECORMENU_TYPE;
@@ -47,7 +47,7 @@ DecorMenu::~DecorMenu(void)
 void
 DecorMenu::handleItemExec(PMenu::Item *item)
 {
-    if (item == NULL) {
+    if (! item) {
         return;
     }
 
@@ -73,7 +73,7 @@ DecorMenu::reload(CfgParser::Entry *section)
     map<string, Theme::PDecorData*>::const_iterator it(_theme->decor_begin());
     for (; it != _theme->decor_end(); ++it) {
         ae.action_list.back().setParamS(it->first);
-        insert(it->first, ae, NULL);
+        insert(Util::to_wide_str(it->first), ae, 0);
     }
 
     buildMenu(); // rebuild the menu
