@@ -1,6 +1,6 @@
 //
 // Frame.cc for pekwm
-// Copyright © 2002-2008 Claes Nästén <me@pekdon.net>
+// Copyright © 2002-2009 Claes Nästén <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -1222,7 +1222,8 @@ Frame::recalcResizeDrag(int nx, int ny, bool left, bool top)
 
     width -= brdr_lr;
     height -= brdr_tb + getTitleHeight();
-    _client->getIncSize(&width, &height, width, height);
+
+    _client->getAspectSize(&width, &height, width, height);
 
     const XSizeHints *hints = _client->getXSizeHints();
     // check so we aren't overriding min or max size
