@@ -1,6 +1,6 @@
 //
 // PScreen.cc for pekwm
-// Copyright © 2003-2008 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2003-2009 Claes Nästén <me{@}pekdon{.}net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -30,6 +30,12 @@ extern "C" {
 #endif // HAVE_XRANDR
 #include <X11/keysym.h> // For XK_ entries
 #include <sys/select.h>
+
+#ifdef DEBUG
+bool xerrors_ignore = false;
+#endif // DEBUG
+
+unsigned int xerrors_count = 0;
 }
 
 #include "PScreen.hh"
@@ -52,6 +58,30 @@ const uint PScreen::MODIFIER_TO_MASK_NUM = sizeof(PScreen::MODIFIER_TO_MASK[0])
 
 PScreen* PScreen::_instance = 0;
 
+
+extern "C" {
+    /**
+      * XError handler, prints error.
+      */
+    static int
+    handleXError(Display *dpy, XErrorEvent *ev)
+    {
+        ++xerrors_count;
+
+#ifdef DEBUG
+        if (xerrors_ignore) {
+            return 0;
+        }
+
+        char error_buf[256];
+        XGetErrorText(dpy, ev->error_code, error_buf, 256);
+        cerr << "XError: " << error_buf << " id: " << ev->resourceid << endl;
+#endif // DEBUG
+
+        return 0;
+    }
+}
+
 //! @brief PScreen::Visual constructor.
 //! @param x_visual X Visual to wrap.
 PScreen::PVisual::PVisual(Visual *x_visual) : _x_visual(x_visual),
@@ -102,6 +132,8 @@ PScreen::PScreen(Display *dpy, bool honour_randr)
     }
     _instance = this;
 
+    XSetErrorHandler(handleXError);
+
     XGrabServer(_dpy);
 
     _fd = ConnectionNumber(dpy);
@@ -140,8 +172,7 @@ PScreen::PScreen(Display *dpy, bool honour_randr)
     }
 
     // Figure out what keys the Num and Scroll Locks are
-    _num_lock = getMaskFromKeycode(XKeysymToKeycode(_dpy, XK_Num_Lock));
-    _scroll_lock = getMaskFromKeycode(XKeysymToKeycode(_dpy, XK_Scroll_Lock));
+    setLockKeys();
 
     XSync(_dpy, false);
     XUngrabServer(_dpy);
@@ -158,6 +189,16 @@ PScreen::~PScreen(void) {
     _instance = 0;
 }
 
+/**
+ * Figure out what keys the Num and Scroll Locks are
+ */
+void
+PScreen::setLockKeys(void)
+{
+    _num_lock = getMaskFromKeycode(XKeysymToKeycode(_dpy, XK_Num_Lock));
+    _scroll_lock = getMaskFromKeycode(XKeysymToKeycode(_dpy, XK_Scroll_Lock));
+}
+
 //! @brief Get next event using select to avoid signal blocking
 //! @param ev Event to fill in.
 //! @return true if event was fetched, else false.
