@@ -1,6 +1,6 @@
 //
 // WindowManager.cc for pekwm
-// Copyright © 2002-2008 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2002-2009 Claes Nästén <me{@}pekdon{.}net>
 //
 // windowmanager.cc for aewm++
 // Copyright (C) 2000 Frank Hale <frankhale@yahoo.com>
@@ -143,27 +143,6 @@ extern "C" {
             break;
         }
     }
-
-    /**
-      * XError handler, prints error.
-      */
-    static int
-    handleXError(Display *dpy, XErrorEvent *e)
-    {
-        if ((e->error_code == BadAccess) &&
-                (e->resourceid == (RootWindow(dpy, DefaultScreen(dpy))))) {
-            cerr << "pekwm: root window unavailable, can't start!" << endl;
-            exit(1);
-#ifdef DEBUG
-        } else {
-            char error_buf[256];
-            XGetErrorText(dpy, e->error_code, error_buf, 256);
-
-            cerr << "XError: " << error_buf << " id: " << e->resourceid << endl;
-#endif // DEBUG
-        }
-        return 0;
-    }
     
 } // extern "C"
 
@@ -173,25 +152,34 @@ extern "C" {
 /**
  * Create window manager instance and run main routine.
  */
-void
-WindowManager::start(const std::string &command_line, const std::string &config_file) {
+WindowManager*
+WindowManager::start(const std::string &command_line,
+                     const std::string &config_file, bool replace)
+{
     if (_instance) {
         delete _instance;
     }
+
+    // Setup window manager
     _instance = new WindowManager(command_line, config_file);
 
-    _instance->setupDisplay();
-
-    _instance->scanWindows();
-    Frame::resetFrameIDs();
-    static_cast<RootWO*>(PWinObj::getRootPWinObj())->setEwmhDesktopNames();
+    if (_instance->setupDisplay(replace)) {
+        _instance->scanWindows();
+        Frame::resetFrameIDs();
+        static_cast<RootWO*>(PWinObj::getRootPWinObj())->setEwmhDesktopNames();
     
-    // add all frames to the MRU list
-    _instance->_mru_list.resize(Frame::frame_size());
-    copy(Frame::frame_begin(), Frame::frame_end (), _instance->_mru_list.begin());
+        // add all frames to the MRU list
+        _instance->_mru_list.resize(Frame::frame_size());
+        copy(Frame::frame_begin(), Frame::frame_end (),
+             _instance->_mru_list.begin());
 
-    _instance->execStartFile();
-    _instance->doEventLoop();
+        _instance->execStartFile();
+    } else {
+        delete _instance;
+        _instance = 0;
+    }
+
+    return _instance;
 }
 
 /**
@@ -205,7 +193,9 @@ WindowManager::destroy(void)
 }
 
 //! @brief Constructor for WindowManager class
-WindowManager::WindowManager(const std::string &command_line, const std::string &config_file) :
+WindowManager::WindowManager(const std::string &command_line,
+                             const std::string &config_file)
+        :
         _screen(0), _screen_resources(0),
         _keygrabber(0),
         _config(0), _color_handler(0),
@@ -219,7 +209,7 @@ WindowManager::WindowManager(const std::string &command_line, const std::string
         _status_window(0), _workspace_indicator(0),
         _command_line(command_line),
         _startup(false), _shutdown(false), _reload(false),
-        _allow_grouping(true), _root_wo(0)
+        _allow_grouping(true), _hint_wo(0), _root_wo(0)
 {
     struct sigaction act;
 
@@ -320,21 +310,29 @@ WindowManager::cleanup(void)
 
     // remove all dockapps
 #ifdef HARBOUR
-    _harbour->removeAllDockApps();
+    if (_harbour) {
+        _harbour->removeAllDockApps();
+    }
 #endif // HARBOUR
 
     // To preserve stacking order when destroying the frames, we go through
     // the PWinObj list from the Workspaces and put all Frames into our own
     // list, then we delete the frames in order.
-    list<Frame*> frame_list;
-    list<PWinObj*>::iterator it_w(_workspaces->begin());
-    for (; it_w != _workspaces->end(); ++it_w) {
-        if ((*it_w)->getType() == PWinObj::WO_FRAME)
-            frame_list.push_back(static_cast<Frame*>(*it_w));
+    if (_workspaces) {
+        list<Frame*> frame_list;
+
+        list<PWinObj*>::iterator it_w(_workspaces->begin());
+        for (; it_w != _workspaces->end(); ++it_w) {
+            if ((*it_w)->getType() == PWinObj::WO_FRAME) {
+                frame_list.push_back(static_cast<Frame*>(*it_w));
+            }
+        }
+
+        // Delete all Frames. This reparents the Clients.
+        for (it_f = frame_list.begin(); it_f != frame_list.end(); ++it_f) {
+            delete *it_f;
+        }
     }
-    // Delete all Frames. This reparents the Clients.
-    for (it_f = frame_list.begin(); it_f != frame_list.end(); ++it_f)
-        delete *it_f;
 
     // Delete all Clients.
     list<Client*> client_list(Client::client_begin(), Client::client_end());
@@ -343,7 +341,9 @@ WindowManager::cleanup(void)
         delete *it_c;
     }
 
-    _keygrabber->ungrabKeys(_screen->getRoot());
+    if (_keygrabber) {
+        _keygrabber->ungrabKeys(_screen->getRoot());
+    }
 
     // destroy screen edge
     screenEdgeDestroy();
@@ -352,9 +352,11 @@ WindowManager::cleanup(void)
     XSetInputFocus(_screen->getDpy(), PointerRoot, RevertToPointerRoot, CurrentTime);
 }
 
-//! @brief
-void
-WindowManager::setupDisplay(void)
+/**
+ * Setup display and claim resources.
+ */
+bool
+WindowManager::setupDisplay(bool replace)
 {
     Display *dpy = XOpenDisplay(0);
     if (! dpy) {
@@ -364,9 +366,23 @@ WindowManager::setupDisplay(void)
         exit(1);
     }
 
-    XSetErrorHandler(handleXError);
-
+    // Setup screen, init atoms and claim the display.
     _screen = new PScreen(dpy, _config->isHonourRandr());
+
+    Atoms::init();
+
+    try {
+        // Create hint window _before_ root window.
+        _hint_wo = new HintWO(dpy, _screen->getRoot(), replace);
+    } catch (string &ex) {
+        return false;
+    }
+
+    // Create root PWinObj
+    _root_wo = new RootWO(dpy, _screen->getRoot());
+    PWinObj::setRootPWinObj(_root_wo);
+
+    // 
     _color_handler = new ColorHandler(dpy);
     _font_handler = new FontHandler();
     _texture_handler = new TextureHandler();
@@ -382,15 +398,6 @@ WindowManager::setupDisplay(void)
     _autoproperties = new AutoProperties();
     _autoproperties->load();
 
-    Atoms::init();
-
-    // Create hint window _before_ root window.
-    _hint_wo = new HintWO(dpy, _screen->getRoot());
-
-    // Create root PWinObj
-    _root_wo = new RootWO(dpy, _screen->getRoot());
-    PWinObj::setRootPWinObj(_root_wo);
-
     _workspaces = new Workspaces(_config->getWorkspaces(), _config->getWorkspacesPerRow());
 
 #ifdef HARBOUR
@@ -405,13 +412,6 @@ WindowManager::setupDisplay(void)
     XDefineCursor(dpy, _screen->getRoot(),
                   _screen_resources->getCursor(ScreenResources::CURSOR_ARROW));
 
-    // select root window events
-    XSelectInput(dpy, _screen->getRoot(),
-                 SubstructureNotifyMask|SubstructureRedirectMask|
-                 ColormapChangeMask|FocusChangeMask|EnterWindowMask|
-                 PropertyChangeMask|
-                 ButtonPressMask|ButtonReleaseMask|ButtonMotionMask);
-
 #ifdef HAVE_XRANDR
     XRRSelectInput(dpy, _screen->getRoot(), RRScreenChangeNotifyMask|RRCrtcChangeNotifyMask);
 #endif // HAVE_XRANDR
@@ -427,6 +427,8 @@ WindowManager::setupDisplay(void)
     // Create screen edge windows
     screenEdgeCreate();
     screenEdgeMapUnmap();
+
+    return true;
 }
 
 //! @brief Goes through the window and creates Clients/DockApps.
@@ -707,10 +709,10 @@ WindowManager::doReloadMouse(void)
  * Reload keygrabber configuration and re-grab keys on all windows.
  */
 void
-WindowManager::doReloadKeygrabber(void)
+WindowManager::doReloadKeygrabber(bool force)
 {
     // Reload the keygrabber
-    if (! _keygrabber->load(_config->getKeyFile())) {
+    if (! _keygrabber->load(_config->getKeyFile(), force)) {
         return;
     }
 
@@ -865,6 +867,12 @@ WindowManager::doEventLoop(void)
                 handleFocusOutEvent(&ev.xfocus);
                 break;
 
+            case SelectionClear:
+                // Another window
+                cerr << " *** INFO: Being replaced by another WM" << endl;
+                _shutdown = true;
+                break;
+
             default:
 #ifdef HAVE_SHAPE
                 if (_screen->hasExtensionShape() && (ev.type == _screen->getEventShape())) {
@@ -1443,7 +1451,8 @@ WindowManager::handleMappingEvent(XMappingEvent *ev)
 {
     if (ev->request == MappingKeyboard || ev->request == MappingModifier) {
         XRefreshKeyboardMapping(ev);
-
+        _screen->setLockKeys();
+        doReloadKeygrabber(true);
     }
 }
 
